@@ -52,6 +52,8 @@ typedef struct {
 
 	GtkWidget    *e_view_folder_checkbutton;
 
+	GtkTooltips  *tooltips;
+
 	gboolean      extract_clicked;
 } DialogData;
 
@@ -66,7 +68,8 @@ destroy_cb (GtkWidget  *widget,
 		window_batch_mode_stop (data->window);
 	}
 
-        g_free (data);
+	g_object_unref (data->tooltips);
+	g_free (data);
 }
 
 
@@ -363,9 +366,6 @@ create_extra_widget (DialogData *data)
 	GtkWidget *label48;
 	GtkWidget *vbox15;
 	GtkWidget *label31;
-	GtkTooltips *tooltips;
-	
-	tooltips = gtk_tooltips_new ();
 	
 	vbox1 = gtk_vbox_new (FALSE, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 0);
@@ -405,7 +405,7 @@ create_extra_widget (DialogData *data)
 	gtk_table_attach (GTK_TABLE (table1), data->e_files_entry, 1, 2, 2, 3,
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			  (GtkAttachOptions) (0), 0, 0);
-	gtk_tooltips_set_tip (tooltips, data->e_files_entry, _("example: *.txt; *.doc"), NULL);
+	gtk_tooltips_set_tip (data->tooltips, data->e_files_entry, _("example: *.txt; *.doc"), NULL);
 	gtk_entry_set_activates_default (GTK_ENTRY (data->e_files_entry), TRUE);
 	
 	data->e_all_radiobutton = gtk_radio_button_new_with_mnemonic (NULL, _("_All files"));
@@ -463,6 +463,7 @@ create_extra_widget (DialogData *data)
 	data->e_password_entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (data->e_password_hbox), data->e_password_entry, TRUE, TRUE, 0);
 	gtk_entry_set_activates_default (GTK_ENTRY (data->e_password_entry), TRUE);
+	gtk_entry_set_visibility (GTK_ENTRY (data->e_password_entry), FALSE);
 	
 	data->e_view_folder_checkbutton = gtk_check_button_new_with_mnemonic (_("_Open destination folder after extraction"));
 	gtk_box_pack_start (GTK_BOX (vbox1), data->e_view_folder_checkbutton, FALSE, FALSE, 0);
@@ -488,6 +489,10 @@ dlg_extract (GtkWidget *widget,
         data->window = window;
 	data->extract_clicked = FALSE;
 
+	data->tooltips = gtk_tooltips_new ();
+	g_object_ref (G_OBJECT (data->tooltips));
+	gtk_object_sink (GTK_OBJECT (data->tooltips));
+
         /* Get the widgets. */
 
 	data->dialog = gtk_file_chooser_dialog_new (
