@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <time.h>
 #include <unistd.h>
 #include <sys/param.h>
@@ -478,21 +479,39 @@ count_chars_to_escape (const char *str,
 }
 
 
-/* escape with backslash the string @str. */
+gboolean
+strchrs (const char *str,
+	 const char *chars)
+{
+	const char *c;
+	for (c = chars; *c != '\0'; c++)
+		if (strchr (str, *c) != NULL)
+			return TRUE;
+	return FALSE;
+}
+
+
 char*
-escape_str (const char *str, 
-	    const char *meta_chars)
+escape_str_common (const char *str, 
+		   const char *meta_chars,
+		   const char  prefix,
+		   const char  postfix)
 {
 	int         meta_chars_n = strlen (meta_chars);
 	char       *escaped;
-	int         i, new_l;
+	int         i, new_l, extra_chars = 0;
 	const char *s;
 	char       *t;
 
 	if (str == NULL) 
 		return NULL;
 
-	new_l = strlen (str) + count_chars_to_escape (str, meta_chars);
+	if (prefix)
+		extra_chars++;
+	if (postfix)
+		extra_chars++;
+
+	new_l = strlen (str) + (count_chars_to_escape (str, meta_chars) * extra_chars);
 	escaped = g_malloc (new_l + 1);
 
 	s = str;
@@ -501,9 +520,11 @@ escape_str (const char *str,
 		gboolean is_bad = FALSE;
 		for (i = 0; (i < meta_chars_n) && !is_bad; i++)
 			is_bad = (*s == meta_chars[i]);
-		if (is_bad)
-			*t++ = '\\';
+		if (is_bad && prefix)
+			*t++ = prefix;
 		*t++ = *s++;
+		if (is_bad && postfix)
+			*t++ = postfix;
 	}
 	*t = 0;
 
@@ -511,91 +532,20 @@ escape_str (const char *str,
 }
 
 
+/* escape with backslash the string @str. */
+char*
+escape_str (const char *str, 
+	    const char *meta_chars)
+{
+	return escape_str_common (str, meta_chars, '\\', 0);
+}
+
+
 /* escape with backslash the file name. */
 char*
 shell_escape (const char *filename)
 {
-	return escape_str (filename, "$\'`\"\\!?* ()[]&|@#:;=-");
-}
-
-
-static gchar *
-get_terminal ()
-{
-	GConfClient *client;
-	gchar *result;
-	gchar *terminal = NULL;
-	gchar *exec_flag = NULL;
-
-	client = gconf_client_get_default ();
-	terminal = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec", NULL);
-	g_object_unref (G_OBJECT (client));
-	
-	if (terminal) 
-		exec_flag = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec_arg", NULL);
-
-	if (terminal == NULL) {
-		char *check;
-
-		check = g_find_program_in_path ("gnome-terminal");
-		if (check != NULL) {
-			terminal = check;
-			/* Note that gnome-terminal takes -x and
-			 * as -e in gnome-terminal is broken we use that. */
-			exec_flag = g_strdup ("-x");
-		} else {
-			if (check == NULL)
-				check = g_find_program_in_path ("nxterm");
-			if (check == NULL)
-				check = g_find_program_in_path ("color-xterm");
-			if (check == NULL)
-				check = g_find_program_in_path ("rxvt");
-			if (check == NULL)
-				check = g_find_program_in_path ("xterm");
-			if (check == NULL)
-				check = g_find_program_in_path ("dtterm");
-			if (check == NULL) {
-				g_warning (_("Cannot find a terminal, using "
-					     "xterm, even if it may not work"));
-				check = g_strdup ("xterm");
-			}
-			terminal = check;
-			exec_flag = g_strdup ("-e");
-		}
-	}
-
-	if (terminal == NULL)
-		return NULL;
-
-	result = g_strconcat (terminal, " ", exec_flag, NULL);
-	return result;
-}
-
-
-gchar *
-application_get_command (const GnomeVFSMimeApplication *app)
-{
-	char *command;
-	
-	if (app->requires_terminal) {
-		char *terminal;
-		char *command_to_exec;
-
-		terminal = get_terminal ();
-		if (terminal == NULL)
-			return NULL;
-
-		command_to_exec = shell_escape (app->command);
-		command = g_strconcat (terminal,
-				       " ",
-				       command_to_exec,
-				       NULL);
-		g_free (terminal);
-		g_free (command_to_exec);
-	} else
-		command = shell_escape (app->command);
-
-	return command;
+	return escape_str (filename, "$\'`\"\\!?* ()[]&|@#:;");
 }
 
 
@@ -619,7 +569,7 @@ match_patterns (char       **patterns,
 		result = g_utf8_fnmatch (patterns[i], string, flags);
 		i++;
 	}
-	
+
 	return (result == 0);
 }
 
@@ -910,6 +860,8 @@ _g_strdup_with_max_size (const char *s,
 const char *
 eat_spaces (const char *line)
 {
+	if (line == NULL)
+		return NULL;
 	while ((*line == ' ') && (*line != 0))
 		line++;
 	return line;
@@ -918,7 +870,7 @@ eat_spaces (const char *line)
 
 char **
 split_line (const char *line, 
-	    int   n_fields)
+	    int         n_fields)
 {
 	char       **fields;
 	const char  *scan, *field_end;
@@ -929,6 +881,10 @@ split_line (const char *line,
 
 	scan = eat_spaces (line);
 	for (i = 0; i < n_fields; i++) {
+		if (scan == NULL) {
+			fields[i] = NULL;
+			continue;
+		}
 		field_end = strchr (scan, ' ');
 		if (field_end != NULL) {
 			fields[i] = g_strndup (scan, field_end - scan);
@@ -947,9 +903,14 @@ get_last_field (const char *line,
 	const char *field;
 	int         i;
 
+	if (line == NULL)
+		return NULL;
+
 	last_field--;
 	field = eat_spaces (line);
 	for (i = 0; i < last_field; i++) {
+		if (field == NULL)
+			return NULL;
 		field = strchr (field, ' ');
 		field = eat_spaces (field);
 	}
@@ -988,3 +949,149 @@ get_temp_work_dir_name (void)
 
 	return result;
 }
+
+
+/* file list utils */
+
+
+gboolean
+file_list__match_pattern (const char *line, 
+			  const char *pattern)
+{
+	const char *l = line, *p = pattern;
+
+	for (; (*p != 0) && (*l != 0); p++, l++) {
+		if (*p != '%') {
+			if (*p != *l)
+				return FALSE;
+		} else {
+			p++;
+			switch (*p) {
+			case 'a':
+				break;
+			case 'n':
+				if (!isdigit (*l)) 
+					return FALSE;
+				break;
+			case 'c':
+				if (!isalpha (*l)) 
+					return FALSE;
+				break;
+			default:
+				return FALSE;
+			}
+		}
+	}
+	
+	return (*p == 0);
+}
+
+
+int
+file_list__get_index_from_pattern (const char *line, 
+				   const char *pattern)
+{
+	int         line_l, pattern_l;
+	const char *l;
+
+	line_l = strlen (line);
+	pattern_l = strlen (pattern);
+
+	if ((pattern_l == 0) || (line_l == 0))
+		return -1;
+
+	for (l = line; *l != 0; l++) 
+		if (file_list__match_pattern (l, pattern))
+			return (l - line);
+	
+	return -1;
+}
+
+
+char*
+file_list__get_next_field (const char *line,
+			   int         start_from,
+			   int         field_n)
+{
+	const char *f_start, *f_end;
+	
+	line = line + start_from;
+	
+	f_start = line;
+	while ((*f_start == ' ') && (*f_start != *line))
+		f_start++;
+	f_end = f_start;
+
+	while ((field_n > 0) && (*f_end != 0)) {
+		if (*f_end == ' ') {
+			field_n--;
+			if (field_n != 0) {
+				while ((*f_end == ' ') && (*f_end != *line))
+					f_end++;
+				f_start = f_end;
+			}
+		} else
+			f_end++;
+	}
+	
+	return g_strndup (f_start, f_end - f_start);
+}
+
+
+char*
+file_list__get_prev_field (const char *line,
+			   int         start_from,
+			   int         field_n)
+{
+	const char *f_start, *f_end;
+
+	f_start = line + start_from - 1;
+	while ((*f_start == ' ') && (*f_start != *line))
+		f_start--;
+	f_end = f_start;
+
+	while ((field_n > 0) && (*f_start != *line)) {
+		if (*f_start == ' ') {
+			field_n--;
+			if (field_n != 0) {
+				while ((*f_start == ' ') && (*f_start != *line))
+					f_start--;
+				f_end = f_start;
+			}
+		} else
+			f_start--;
+	}
+
+	return g_strndup (f_start + 1, f_end - f_start);
+}
+
+
+char *
+str_substitute (const char *str,
+		const char *from_str,
+		const char *to_str)
+{
+	char    **tokens;
+	int       i;
+	GString  *gstr;
+
+	if (str == NULL)
+		return NULL;
+
+	if (from_str == NULL)
+		return g_strdup (str);
+
+	if (strcmp (str, from_str) == 0)
+		return g_strdup (to_str);
+
+	tokens = g_strsplit (str, from_str, -1);
+
+	gstr = g_string_new (NULL);
+	for (i = 0; tokens[i] != NULL; i++) {
+		gstr = g_string_append (gstr, tokens[i]);
+		if ((to_str != NULL) && (tokens[i+1] != NULL))
+			gstr = g_string_append (gstr, to_str);
+	}
+
+	return g_string_free (gstr, FALSE);
+}
